@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 #    Copyright (C) 2015 Yahoo! Inc. All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -21,45 +19,73 @@ import heapq
 import inspect
 import logging
 import math
+import random
 import threading
 
-# For: https://wiki.openstack.org/wiki/Security/Projects/Bandit
-from random import SystemRandom as random
-
 from concurrent import futures
-import six
+try:
+    import prettytable
+except ImportError:
+    prettytable = None
 
 import futurist
 from futurist import _utils as utils
 
 LOG = logging.getLogger(__name__)
 
+
+class NeverAgain(Exception):
+    """Exception to raise to stop further periodic calls for a function.
+
+    When you want a function never run again you can throw this from
+    you periodic function and that will signify to the execution framework
+    to remove that function (and never run it again).
+    """
+
+
 _REQUIRED_ATTRS = ('_is_periodic', '_periodic_spacing',
                    '_periodic_run_immediately')
 
+_DEFAULT_COLS = ('Name', 'Active', 'Periodicity', 'Runs in',
+                 'Runs', 'Failures', 'Successes', 'Stop Requested',
+                 'Average elapsed', 'Average elapsed waiting')
+
 # Constants that are used to determine what 'kind' the current callback
 # is being ran as.
 PERIODIC = 'periodic'
 IMMEDIATE = 'immediate'
 
 
+class Work(collections.namedtuple("Work",
+                                  ['name', 'callback', 'args', 'kwargs'])):
+    """Named unit of work that can be periodically scheduled and watched."""
+
+    def __call__(self):
+        return self.callback(*self.args, **self.kwargs)
+
+
 class Watcher(object):
-    """A **read-only** object representing a periodics callbacks activities."""
+    """A **read-only** object representing a periodic callback's activities."""
 
-    _REPR_MSG_TPL = ("<Watcher object at 0x%(ident)x "
-                     "("
-                     "runs=%(runs)s,"
-                     " successes=%(successes)s,"
-                     " failures=%(failures)s,"
-                     " elapsed=%(elapsed)0.2f,"
-                     " elapsed_waiting=%(elapsed_waiting)0.2f"
-                     ")>")
-
-    def __init__(self, metrics):
+    def __init__(self, metrics, work):
         self._metrics = metrics
+        self._work = work
 
     def __repr__(self):
-        return self._REPR_MSG_TPL % dict(ident=id(self), **self._metrics)
+        return ("<Watcher(metrics=%(metrics)s, work=%(work)s)"
+                " object at 0x%(ident)x>") % dict(ident=id(self),
+                                                  work=self._work,
+                                                  metrics=self._metrics)
+
+    @property
+    def requested_stop(self):
+        """If the work unit being ran has requested to be stopped."""
+        return self._metrics['requested_stop']
+
+    @property
+    def work(self):
+        """**Read-only** named work tuple this object watches."""
+        return self._work
 
     @property
     def runs(self):
@@ -138,7 +164,7 @@ def periodic(spacing, run_immediately=False, enabled=True):
     :type enabled: boolean
     """
 
-    if spacing <= 0:
+    if spacing <= 0 and enabled:
         raise ValueError("Periodicity/spacing must be greater than"
                          " zero instead of %s" % spacing)
 
@@ -147,7 +173,7 @@ def periodic(spacing, run_immediately=False, enabled=True):
         f._periodic_spacing = spacing
         f._periodic_run_immediately = run_immediately
 
-        @six.wraps(f)
+        @functools.wraps(f)
         def decorator(*args, **kwargs):
             return f(*args, **kwargs)
 
@@ -167,12 +193,13 @@ def _add_jitter(max_percent_jitter):
                          " equal to 0.0 and less than or equal to 1.0")
 
     def wrapper(func):
+        rnd = random.SystemRandom()
 
-        @six.wraps(func)
-        def decorator(cb, metrics, now=None):
-            next_run = func(cb, metrics, now=now)
+        @functools.wraps(func)
+        def decorator(cb, started_at, finished_at, metrics):
+            next_run = func(cb, started_at, finished_at, metrics)
             how_often = cb._periodic_spacing
-            jitter = how_often * (random.random() * max_percent_jitter)
+            jitter = how_often * (rnd.random() * max_percent_jitter)
             return next_run + jitter
 
         decorator.__name__ += "_with_jitter"
@@ -231,6 +258,12 @@ class _Schedule(object):
     def __len__(self):
         return len(self._ordering)
 
+    def fetch_next_run(self, index):
+        for (next_run, a_index) in self._ordering:
+            if a_index == index:
+                return next_run
+        return None
+
     def pop(self):
         return heapq.heappop(self._ordering)
 
@@ -245,45 +278,31 @@ def _on_failure_log(log, cb, kind, spacing, exc_info, traceback=None):
                   " seconds):\n%s", kind, cb_name, spacing, traceback)
 
 
-def _run_callback_retain(now_func, cb, *args, **kwargs):
-    # NOTE(harlowja): this needs to be a module level function so that the
-    # process pool execution can locate it (it can't be a lambda or method
-    # local function because it won't be able to find those).
-    failure = None
-    started_at = now_func()
-    try:
-        cb(*args, **kwargs)
-    except Exception:
-        # Until https://bugs.python.org/issue24451 is merged we have to
-        # capture and return the failure, so that we can have reliable
-        # timing information.
-        failure = utils.Failure(True)
-    finished_at = now_func()
-    return (started_at, finished_at, failure)
+class _Runner(object):
+    def __init__(self, now_func, retain_traceback=True):
+        self.now_func = now_func
+        self.retain_traceback = retain_traceback
+
+    def run(self, work):
+        failure = None
+        started_at = self.now_func()
+        try:
+            work()
+        except Exception:
+            # Until https://bugs.python.org/issue24451 is merged we have to
+            # capture and return the failure, so that we can have reliable
+            # timing information.
+            failure = utils.Failure(self.retain_traceback)
+        finished_at = self.now_func()
+        return (started_at, finished_at, failure)
 
 
-def _run_callback_no_retain(now_func, cb, *args, **kwargs):
-    # NOTE(harlowja): this needs to be a module level function so that the
-    # process pool execution can locate it (it can't be a lambda or method
-    # local function because it won't be able to find those).
-    failure = None
-    started_at = now_func()
-    try:
-        cb(*args, **kwargs)
-    except Exception:
-        # Until https://bugs.python.org/issue24451 is merged we have to
-        # capture and return the failure, so that we can have reliable
-        # timing information.
-        failure = utils.Failure(False)
-    finished_at = now_func()
-    return (started_at, finished_at, failure)
-
-
-def _build(now_func, callables, next_run_scheduler):
+def _build(now_func, works, next_run_scheduler):
     schedule = _Schedule()
     now = None
     immediates = collections.deque()
-    for index, (cb, _cb_name, args, kwargs) in enumerate(callables):
+    for index, work in enumerate(works):
+        cb = work.callback
         if cb._periodic_run_immediately:
             immediates.append(index)
         else:
@@ -339,6 +358,7 @@ class PeriodicWorker(object):
         'elapsed_waiting': 0,
         'failures': 0,
         'successes': 0,
+        'requested_stop': False,
     }
 
     # When scheduling fails temporary, use a random delay between 0.9-1.1 sec.
@@ -438,18 +458,19 @@ class PeriodicWorker(object):
                            four positional arguments and one keyword
                            argument, the first positional argument being the
                            callable that failed, the second being the type
-                           of activity under which it failed (IMMEDIATE or
-                           PERIODIC), the third being the spacing that the
-                           callable runs at and the fourth `exc_info` tuple
-                           of the failure. The keyword argument 'traceback'
+                           of activity under which it failed (``IMMEDIATE`` or
+                           ``PERIODIC``), the third being the spacing that the
+                           callable runs at and the fourth ``exc_info`` tuple
+                           of the failure. The keyword argument ``traceback``
                            will also be provided that may be be a string
                            that caused the failure (this is required for
                            executors which run out of process, as those can not
-                           transfer stack frames across process boundaries); if
-                           no callable is provided then a default failure
-                           logging function will be used instead, do note that
+                           *currently* transfer stack frames across process
+                           boundaries); if no callable is provided then a
+                           default failure logging function will be used
+                           instead (do note that
                            any user provided callable should not raise
-                           exceptions on being called
+                           exceptions on being called)
         :type on_failure: callable
         :param args: positional arguments to be passed to all callables
         :type args: tuple
@@ -461,7 +482,7 @@ class PeriodicWorker(object):
             for (name, member) in inspect.getmembers(obj):
                 if name.startswith("_") and exclude_hidden:
                     continue
-                if six.callable(member):
+                if callable(member):
                     missing_attrs = _check_attrs(member)
                     if not missing_attrs:
                         callables.append((member, args, kwargs))
@@ -518,29 +539,33 @@ class PeriodicWorker(object):
                            four positional arguments and one keyword
                            argument, the first positional argument being the
                            callable that failed, the second being the type
-                           of activity under which it failed (IMMEDIATE or
-                           PERIODIC), the third being the spacing that the
-                           callable runs at and the fourth `exc_info` tuple
-                           of the failure. The keyword argument 'traceback'
+                           of activity under which it failed (``IMMEDIATE`` or
+                           ``PERIODIC``), the third being the spacing that the
+                           callable runs at and the fourth ``exc_info`` tuple
+                           of the failure. The keyword argument ``traceback``
                            will also be provided that may be be a string
                            that caused the failure (this is required for
                            executors which run out of process, as those can not
-                           transfer stack frames across process boundaries); if
-                           no callable is provided then a default failure
-                           logging function will be used instead, do note that
+                           *currently* transfer stack frames across process
+                           boundaries); if no callable is provided then a
+                           default failure logging function will be used
+                           instead (do note that
                            any user provided callable should not raise
-                           exceptions on being called
+                           exceptions on being called)
         :type on_failure: callable
         """
+        if on_failure is not None and not callable(on_failure):
+            raise ValueError("On failure callback %r must be"
+                             " callable" % on_failure)
         self._tombstone = event_cls()
         self._waiter = cond_cls()
         self._dead = event_cls()
         self._active = event_cls()
         self._cond_cls = cond_cls
         self._watchers = []
-        self._callables = []
+        self._works = []
         for (cb, args, kwargs) in callables:
-            if not six.callable(cb):
+            if not callable(cb):
                 raise ValueError("Periodic callback %r must be callable" % cb)
             missing_attrs = _check_attrs(cb)
             if missing_attrs:
@@ -552,11 +577,11 @@ class PeriodicWorker(object):
                 if args is None:
                     args = self._NO_OP_ARGS
                 if kwargs is None:
-                    kwargs = self._NO_OP_KWARGS
-                cb_name = utils.get_callback_name(cb)
+                    kwargs = self._NO_OP_KWARGS.copy()
                 cb_metrics = self._INITIAL_METRICS.copy()
-                watcher = Watcher(cb_metrics)
-                self._callables.append((cb, cb_name, args, kwargs))
+                work = Work(utils.get_callback_name(cb), cb, args, kwargs)
+                watcher = Watcher(cb_metrics, work)
+                self._works.append(work)
                 self._watchers.append((cb_metrics, watcher))
         try:
             strategy = self.BUILT_IN_STRATEGIES[schedule_strategy]
@@ -568,21 +593,24 @@ class PeriodicWorker(object):
                              " %s selectable strategies"
                              % (schedule_strategy, valid_strategies))
         self._immediates, self._schedule = _build(
-            now_func, self._callables, self._initial_schedule_strategy)
+            now_func, self._works, self._initial_schedule_strategy)
         self._log = log or LOG
         if executor_factory is None:
             executor_factory = lambda: futurist.SynchronousExecutor()
-        self._on_failure = functools.partial(_on_failure_log, self._log)
+        if on_failure is None:
+            on_failure = functools.partial(_on_failure_log, self._log)
+        self._on_failure = on_failure
         self._executor_factory = executor_factory
         self._now_func = now_func
 
     def __len__(self):
-        """How many callables are currently active."""
-        return len(self._callables)
+        """How many callables/periodic work units are currently active."""
+        return len(self._works)
 
-    def _run(self, executor, runner):
+    def _run(self, executor, runner, auto_stop_when_empty):
         """Main worker run loop."""
         barrier = utils.Barrier(cond_cls=self._cond_cls)
+        rnd = random.SystemRandom()
 
         def _process_scheduled():
             # Figure out when we should run next (by selecting the
@@ -606,31 +634,27 @@ class PeriodicWorker(object):
                 when_next = next_run - now
                 if when_next <= 0:
                     # Run & schedule its next execution.
-                    cb, cb_name, args, kwargs = self._callables[index]
-                    self._log.debug("Submitting periodic function '%s'",
-                                    cb_name)
+                    work = self._works[index]
+                    self._log.debug("Submitting periodic"
+                                    " callback '%s'", work.name)
                     try:
-                        fut = executor.submit(runner,
-                                              self._now_func,
-                                              cb, *args, **kwargs)
+                        fut = executor.submit(runner.run, work)
                     except _SCHEDULE_RETRY_EXCEPTIONS as exc:
                         # Restart after a short delay
                         delay = (self._RESCHEDULE_DELAY +
-                                 random().random() * self._RESCHEDULE_JITTER)
-                        self._log.error("Failed to submit periodic function "
+                                 rnd.random() * self._RESCHEDULE_JITTER)
+                        self._log.error("Failed to submit periodic callback "
                                         "'%s', retrying after %.2f sec. "
                                         "Error: %s",
-                                        cb_name, delay, exc)
+                                        work.name, delay, exc)
                         self._schedule.push(self._now_func() + delay,
                                             index)
                     else:
                         barrier.incr()
                         fut.add_done_callback(functools.partial(_on_done,
                                                                 PERIODIC,
-                                                                cb, cb_name,
-                                                                index,
+                                                                work, index,
                                                                 submitted_at))
-                        fut.add_done_callback(lambda _fut: barrier.decr())
                 else:
                     # Gotta wait...
                     self._schedule.push(next_run, index)
@@ -638,50 +662,76 @@ class PeriodicWorker(object):
                     self._waiter.wait(when_next)
 
         def _process_immediates():
-            try:
-                index = self._immediates.popleft()
-            except IndexError:
-                pass
-            else:
-                cb, cb_name, args, kwargs = self._callables[index]
-                submitted_at = self._now_func()
-                self._log.debug("Submitting immediate function '%s'", cb_name)
+            with self._waiter:
                 try:
-                    fut = executor.submit(runner, self._now_func,
-                                          cb, *args, **kwargs)
-                except _SCHEDULE_RETRY_EXCEPTIONS as exc:
-                    self._log.error("Failed to submit immediate function "
-                                    "'%s', retrying. Error: %s", cb_name, exc)
-                    # Restart as soon as possible
-                    self._immediates.append(index)
+                    index = self._immediates.popleft()
+                except IndexError:
+                    pass
                 else:
-                    barrier.incr()
-                    fut.add_done_callback(functools.partial(_on_done,
-                                                            IMMEDIATE,
-                                                            cb, cb_name,
-                                                            index,
-                                                            submitted_at))
-                    fut.add_done_callback(lambda _fut: barrier.decr())
+                    work = self._works[index]
+                    submitted_at = self._now_func()
+                    self._log.debug("Submitting immediate"
+                                    " callback '%s'", work.name)
+                    try:
+                        fut = executor.submit(runner.run, work)
+                    except _SCHEDULE_RETRY_EXCEPTIONS as exc:
+                        self._log.error("Failed to submit immediate callback "
+                                        "'%s', retrying. Error: %s", work.name,
+                                        exc)
+                        # Restart as soon as possible
+                        self._immediates.append(index)
+                    else:
+                        barrier.incr()
+                        fut.add_done_callback(functools.partial(_on_done,
+                                                                IMMEDIATE,
+                                                                work, index,
+                                                                submitted_at))
 
-        def _on_done(kind, cb, cb_name, index, submitted_at, fut):
+        def _on_done(kind, work, index, submitted_at, fut):
+            cb = work.callback
             started_at, finished_at, failure = fut.result()
             cb_metrics, _watcher = self._watchers[index]
             cb_metrics['runs'] += 1
+            schedule_again = True
             if failure is not None:
-                cb_metrics['failures'] += 1
-                self._on_failure(cb, kind, cb._periodic_spacing,
-                                 failure.exc_info, traceback=failure.traceback)
+                if not issubclass(failure.exc_type, NeverAgain):
+                    cb_metrics['failures'] += 1
+                    try:
+                        self._on_failure(cb, kind, cb._periodic_spacing,
+                                         failure.exc_info,
+                                         traceback=failure.traceback)
+                    except Exception as exc:
+                        self._log.error("On failure callback %r raised an"
+                                        " unhandled exception. Error: %s",
+                                        self._on_failure, exc)
+                else:
+                    cb_metrics['successes'] += 1
+                    schedule_again = False
+                    self._log.debug("Periodic callback '%s' raised "
+                                    "'NeverAgain' "
+                                    "exception, stopping any further "
+                                    "execution of it.", work.name)
             else:
                 cb_metrics['successes'] += 1
             elapsed = max(0, finished_at - started_at)
             elapsed_waiting = max(0, started_at - submitted_at)
             cb_metrics['elapsed'] += elapsed
             cb_metrics['elapsed_waiting'] += elapsed_waiting
-            next_run = self._schedule_strategy(cb,
-                                               started_at, finished_at,
-                                               cb_metrics)
             with self._waiter:
-                self._schedule.push(next_run, index)
+                with barrier.decr_cm() as am_left:
+                    if schedule_again:
+                        next_run = self._schedule_strategy(cb, started_at,
+                                                           finished_at,
+                                                           cb_metrics)
+                        self._schedule.push(next_run, index)
+                    else:
+                        cb_metrics['requested_stop'] = True
+                        if (am_left <= 0 and
+                                len(self._immediates) == 0 and
+                                len(self._schedule) == 0 and
+                                auto_stop_when_empty):
+                            # Guess nothing left to do, goodbye...
+                            self._tombstone.set()
                 self._waiter.notify_all()
 
         try:
@@ -695,22 +745,74 @@ class PeriodicWorker(object):
         # TODO(harlowja): this may be to verbose for people?
         if not self._log.isEnabledFor(logging.DEBUG):
             return
-        watcher_it = self.iter_watchers()
-        for index, watcher in enumerate(watcher_it):
-            cb, cb_name, _args, _kwargs = self._callables[index]
-            self._log.debug("Stopped running callback[%s] '%s' periodically:",
-                            index, cb_name)
-            self._log.debug("  Periodicity = %ss", cb._periodic_spacing)
-            self._log.debug("  Runs = %s", watcher.runs)
-            self._log.debug("  Failures = %s", watcher.failures)
-            self._log.debug("  Successes = %s", watcher.successes)
+        cols = list(_DEFAULT_COLS)
+        for c in ['Runs in', 'Active', 'Periodicity']:
+            cols.remove(c)
+        self._log.debug(
+            "Stopped running %s callbacks:\n%s",
+            len(self._works),
+            self.pformat(columns=cols) if prettytable
+            else "statistics not available, PrettyTable missing"
+        )
+
+    def pformat(self, columns=_DEFAULT_COLS):
+        if prettytable is None:
+            raise ImportError(
+                "PrettyTable is required to use the pformat method")
+        # Convert to a list to ensure we maintain the same order when used
+        # further in this function (since order will matter)...
+        if not isinstance(columns, (list, tuple)):
+            columns = list(columns)
+        if not columns:
+            raise ValueError("At least one of %s columns must"
+                             " be provided" % (set(_DEFAULT_COLS)))
+        for c in columns:
+            if c not in _DEFAULT_COLS:
+                raise ValueError("Unknown column '%s', valid column names"
+                                 " are %s" % (c, set(_DEFAULT_COLS)))
+        tbl_rows = []
+        now = self._now_func()
+        for index, work in enumerate(self._works):
+            _cb_metrics, watcher = self._watchers[index]
+            next_run = self._schedule.fetch_next_run(index)
+            if watcher.requested_stop:
+                active = False
+                runs_in = 'n/a'
+            elif next_run is None:
+                active = True
+                runs_in = 'n/a'
+            else:
+                active = False
+                runs_in = "%0.4fs" % (max(0.0, next_run - now))
+            cb_row = {
+                'Name': work.name,
+                'Active': active,
+                'Periodicity': work.callback._periodic_spacing,
+                'Runs': watcher.runs,
+                'Runs in': runs_in,
+                'Failures': watcher.failures,
+                'Successes': watcher.successes,
+                'Stop Requested': watcher.requested_stop,
+            }
             try:
-                self._log.debug("  Average elapsed = %0.4fs",
-                                watcher.average_elapsed)
-                self._log.debug("  Average elapsed waiting = %0.4fs",
-                                watcher.average_elapsed_waiting)
+                cb_row_avgs = [
+                    "%0.4fs" % watcher.average_elapsed,
+                    "%0.4fs" % watcher.average_elapsed_waiting,
+                ]
             except ZeroDivisionError:
-                pass
+                cb_row_avgs = ['.', '.']
+            cb_row['Average elapsed'] = cb_row_avgs[0]
+            cb_row['Average elapsed waiting'] = cb_row_avgs[1]
+            tbl_rows.append(cb_row)
+        # Now form the table, but use only the columns that the caller
+        # asked for (and in the order they asked for...)
+        tbl = prettytable.PrettyTable(columns)
+        for cb_row in tbl_rows:
+            tbl_row = []
+            for c in columns:
+                tbl_row.append(cb_row[c])
+            tbl.add_row(tbl_row)
+        return tbl.get_string()
 
     def add(self, cb, *args, **kwargs):
         """Adds a new periodic callback to the current worker.
@@ -723,7 +825,7 @@ class PeriodicWorker(object):
                    with the :py:func:`.periodic` decorator
         :type cb: callable
         """
-        if not six.callable(cb):
+        if not callable(cb):
             raise ValueError("Periodic callback %r must be callable" % cb)
         missing_attrs = _check_attrs(cb)
         if missing_attrs:
@@ -733,11 +835,11 @@ class PeriodicWorker(object):
             return None
         now = self._now_func()
         with self._waiter:
-            cb_index = len(self._callables)
-            cb_name = utils.get_callback_name(cb)
+            cb_index = len(self._works)
             cb_metrics = self._INITIAL_METRICS.copy()
-            watcher = Watcher(cb_metrics)
-            self._callables.append((cb, cb_name, args, kwargs))
+            work = Work(utils.get_callback_name(cb), cb, args, kwargs)
+            watcher = Watcher(cb_metrics, work)
+            self._works.append(work)
             self._watchers.append((cb_metrics, watcher))
             if cb._periodic_run_immediately:
                 self._immediates.append(cb_index)
@@ -747,7 +849,7 @@ class PeriodicWorker(object):
             self._waiter.notify_all()
             return watcher
 
-    def start(self, allow_empty=False):
+    def start(self, allow_empty=False, auto_stop_when_empty=False):
         """Starts running (will not return until :py:meth:`.stop` is called).
 
         :param allow_empty: instead of running with no callbacks raise when
@@ -758,10 +860,20 @@ class PeriodicWorker(object):
                             sleep (until either stopped or callbacks are
                             added)
         :type allow_empty: bool
+        :param auto_stop_when_empty: when the provided periodic functions have
+                                     all exited and this is false then the
+                                     thread responsible for executing those
+                                     methods will just spin/idle waiting for
+                                     a new periodic function to be added;
+                                     switching it to true will make this
+                                     idling not happen (and instead when no
+                                     more periodic work exists then the
+                                     calling thread will just return).
+        :type auto_stop_when_empty: bool
         """
-        if not self._callables and not allow_empty:
+        if not self._works and not allow_empty:
             raise RuntimeError("A periodic worker can not start"
-                               " without any callables")
+                               " without any callables to process")
         if self._active.is_set():
             raise RuntimeError("A periodic worker can not be started"
                                " twice")
@@ -774,13 +886,13 @@ class PeriodicWorker(object):
             # Pickling a traceback will not work, so do not try to do it...
             #
             # Avoids 'TypeError: can't pickle traceback objects'
-            runner = _run_callback_no_retain
+            runner = _Runner(self._now_func, retain_traceback=False)
         else:
-            runner = _run_callback_retain
+            runner = _Runner(self._now_func, retain_traceback=True)
         self._dead.clear()
         self._active.set()
         try:
-            self._run(executor, runner)
+            self._run(executor, runner, auto_stop_when_empty)
         finally:
             if getattr(self._executor_factory, 'shutdown', True):
                 executor.shutdown()
@@ -804,13 +916,13 @@ class PeriodicWorker(object):
         self._tombstone.clear()
         self._dead.clear()
         for cb_metrics, _watcher in self._watchers:
-            for k in list(six.iterkeys(cb_metrics)):
+            for k in list(cb_metrics):
                 # NOTE(harlowja): mutate the original dictionaries keys
                 # so that the watcher (which references the same dictionary
                 # keys) is able to see those changes.
                 cb_metrics[k] = 0
         self._immediates, self._schedule = _build(
-            self._now_func, self._callables, self._initial_schedule_strategy)
+            self._now_func, self._works, self._initial_schedule_strategy)
 
     def wait(self, timeout=None):
         """Waits for the :py:meth:`.start` method to gracefully exit.
